@@ -9,11 +9,11 @@
 use uu_base32::base_common;
 pub use uu_base32::uu_app;
 
-use uucore::{encoding::Format, error::UResult, help_section, help_usage};
+use uucore::{encoding::Format, error::UResult, help_about, help_usage};
 
 use std::io::{stdin, Read};
 
-const ABOUT: &str = help_section!("about", "base64.md");
+const ABOUT: &str = help_about!("base64.md");
 const USAGE: &str = help_usage!("base64.md");
 
 #[uucore::main]
