@@ -9,6 +9,7 @@
 
 use bstr::io::BufReadExt;
 use clap::{crate_version, Arg, ArgAction, Command};
+use is_terminal::IsTerminal;
 use std::fs::File;
 use std::io::{stdin, stdout, BufReader, BufWriter, Read, Write};
 use std::path::Path;
@@ -16,101 +17,16 @@ use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError};
 
 use self::searcher::Searcher;
-use self::whitespace_searcher::WhitespaceSearcher;
+use matcher::{ExactMatcher, Matcher, WhitespaceMatcher};
 use uucore::ranges::Range;
-use uucore::{format_usage, show, show_error, show_if_err};
+use uucore::{format_usage, help_about, help_section, help_usage, show, show_error, show_if_err};
 
+mod matcher;
 mod searcher;
-mod whitespace_searcher;
 
-static USAGE: &str =
-    "{} [-d|-w] [-s] [-z] [--output-delimiter] ((-f|-b|-c) {{sequence}}) {{sourcefile}}+";
-static ABOUT: &str =
-    "Prints specified byte or field columns from each line of stdin or the input files";
-static LONG_HELP: &str = "
- Each call must specify a mode (what to use for columns),
- a sequence (which columns to print), and provide a data source
-
- Specifying a mode
-
-    Use --bytes (-b) or --characters (-c) to specify byte mode
-
-    Use --fields (-f) to specify field mode, where each line is broken into
-    fields identified by a delimiter character. For example for a typical CSV
-    you could use this in combination with setting comma as the delimiter
-
- Specifying a sequence
-
-    A sequence is a group of 1 or more numbers or inclusive ranges separated
-    by a commas.
-
-    cut -f 2,5-7 some_file.txt
-    will display the 2nd, 5th, 6th, and 7th field for each source line
-
-    Ranges can extend to the end of the row by excluding the the second number
-
-    cut -f 3- some_file.txt
-    will display the 3rd field and all fields after for each source line
-
-    The first number of a range can be excluded, and this is effectively the
-    same as using 1 as the first number: it causes the range to begin at the
-    first column. Ranges can also display a single column
-
-    cut -f 1,3-5 some_file.txt
-    will display the 1st, 3rd, 4th, and 5th field for each source line
-
-    The --complement option, when used, inverts the effect of the sequence
-
-    cut --complement -f 4-6 some_file.txt
-    will display the every field but the 4th, 5th, and 6th
-
- Specifying a data source
-
-    If no sourcefile arguments are specified, stdin is used as the source of
-    lines to print
-
-    If sourcefile arguments are specified, stdin is ignored and all files are
-    read in consecutively if a sourcefile is not successfully read, a warning
-    will print to stderr, and the eventual status code will be 1, but cut
-    will continue to read through proceeding sourcefiles
-
-    To print columns from both STDIN and a file argument, use - (dash) as a
-    sourcefile argument to represent stdin.
-
- Field Mode options
-
-    The fields in each line are identified by a delimiter (separator)
-
-    Set the delimiter
-        Set the delimiter which separates fields in the file using the
-        --delimiter (-d) option. Setting the delimiter is optional.
-        If not set, a default delimiter of Tab will be used.
-
-        If the -w option is provided, fields will be separated by any number
-        of whitespace characters (Space and Tab). The output delimiter will
-        be a Tab unless explicitly specified. Only one of -d or -w option can be specified.
-        This is an extension adopted from FreeBSD.
-
-    Optionally Filter based on delimiter
-        If the --only-delimited (-s) flag is provided, only lines which
-        contain the delimiter will be printed
-
-    Replace the delimiter
-        If the --output-delimiter option is provided, the argument used for
-        it will replace the delimiter character in each line printed. This is
-        useful for transforming tabular data - e.g. to convert a CSV to a
-        TSV (tab-separated file)
-
- Line endings
-
-    When the --zero-terminated (-z) option is used, cut sees \\0 (null) as the
-    'line ending' character (both for the purposes of reading lines and
-    separating printed lines) instead of \\n (newline). This is useful for
-    tabular data where some of the cells may contain newlines
-
-    echo 'ab\\0cd' | cut -z -c 1
-    will result in 'a\\0c\\0'
-";
+const USAGE: &str = help_usage!("cut.md");
+const ABOUT: &str = help_about!("cut.md");
+const AFTER_HELP: &str = help_section!("after help", "cut.md");
 
 struct Options {
     out_delim: Option<String>,
@@ -136,7 +52,7 @@ enum Mode {
 }
 
 fn stdout_writer() -> Box<dyn Write> {
-    if atty::is(atty::Stream::Stdout) {
+    if std::io::stdout().is_terminal() {
         Box::new(stdout())
     } else {
         Box::new(BufWriter::new(stdout())) as Box<dyn Write>
@@ -188,23 +104,22 @@ fn cut_bytes<R: Read>(reader: R, ranges: &[Range], opts: &Options) -> UResult<()
     Ok(())
 }
 
-#[allow(clippy::cognitive_complexity)]
-fn cut_fields_delimiter<R: Read>(
+// Output delimiter is explicitly specified
+fn cut_fields_explicit_out_delim<R: Read, M: Matcher>(
     reader: R,
+    matcher: &M,
     ranges: &[Range],
-    delim: &str,
     only_delimited: bool,
     newline_char: u8,
     out_delim: &str,
 ) -> UResult<()> {
     let mut buf_in = BufReader::new(reader);
     let mut out = stdout_writer();
-    let input_delim_len = delim.len();
 
     let result = buf_in.for_byte_record_with_terminator(newline_char, |line| {
         let mut fields_pos = 1;
         let mut low_idx = 0;
-        let mut delim_search = Searcher::new(line, delim.as_bytes()).peekable();
+        let mut delim_search = Searcher::new(matcher, line).peekable();
         let mut print_delim = false;
 
         if delim_search.peek().is_none() {
@@ -218,85 +133,6 @@ fn cut_fields_delimiter<R: Read>(
             return Ok(true);
         }
 
-        for &Range { low, high } in ranges {
-            if low - fields_pos > 0 {
-                low_idx = match delim_search.nth(low - fields_pos - 1) {
-                    Some(index) => index + input_delim_len,
-                    None => break,
-                };
-            }
-
-            for _ in 0..=high - low {
-                if print_delim {
-                    out.write_all(out_delim.as_bytes())?;
-                } else {
-                    print_delim = true;
-                }
-
-                match delim_search.next() {
-                    Some(high_idx) => {
-                        let segment = &line[low_idx..high_idx];
-
-                        out.write_all(segment)?;
-
-                        low_idx = high_idx + input_delim_len;
-                        fields_pos = high + 1;
-                    }
-                    None => {
-                        let segment = &line[low_idx..];
-
-                        out.write_all(segment)?;
-
-                        if line[line.len() - 1] == newline_char {
-                            return Ok(true);
-                        }
-                        break;
-                    }
-                }
-            }
-        }
-
-        out.write_all(&[newline_char])?;
-        Ok(true)
-    });
-
-    if let Err(e) = result {
-        return Err(USimpleError::new(1, e.to_string()));
-    }
-
-    Ok(())
-}
-
-fn cut_fields_whitespace<R: Read>(
-    reader: R,
-    ranges: &[Range],
-    only_delimited: bool,
-    newline_char: u8,
-    out_delim: &str,
-) -> UResult<()> {
-    let mut buf_in = BufReader::new(reader);
-    let mut out = stdout_writer();
-
-    let result = buf_in.for_byte_record_with_terminator(newline_char, |line| {
-        let mut fields_pos = 1;
-        let mut low_idx = 0;
-        let mut delim_search = WhitespaceSearcher::new(line).peekable();
-        let mut print_delim = false;
-
-        if delim_search.peek().is_none() {
-            if !only_delimited {
-                out.write_all(line)?;
-                if line[line.len() - 1] != newline_char {
-                    out.write_all(&[newline_char])?;
-                }
-            }
-
-            return Ok(true);
-        }
-        // The logic is identical to `cut_fields_delimiter` function above, which uses
-        // `Searcher` that iterates over and returns the first position of the delimiter character.
-        // The main difference is that `WhitespaceSearcher` returns a pair of the first and last
-        // delimiter character positions, since each delimiter sequence length can vary.
         for &Range { low, high } in ranges {
             if low - fields_pos > 0 {
                 // current field is not in the range, so jump to the field corresponding to the
@@ -317,7 +153,7 @@ fn cut_fields_whitespace<R: Read>(
                 }
 
                 match delim_search.next() {
-                    // print the current field up to the next whitespace
+                    // print the current field up to the next field delim
                     Some((first, last)) => {
                         let segment = &line[low_idx..first];
 
@@ -352,93 +188,110 @@ fn cut_fields_whitespace<R: Read>(
     Ok(())
 }
 
+// Output delimiter is the same as input delimiter
+fn cut_fields_implicit_out_delim<R: Read, M: Matcher>(
+    reader: R,
+    matcher: &M,
+    ranges: &[Range],
+    only_delimited: bool,
+    newline_char: u8,
+) -> UResult<()> {
+    let mut buf_in = BufReader::new(reader);
+    let mut out = stdout_writer();
+
+    let result = buf_in.for_byte_record_with_terminator(newline_char, |line| {
+        let mut fields_pos = 1;
+        let mut low_idx = 0;
+        let mut delim_search = Searcher::new(matcher, line).peekable();
+        let mut print_delim = false;
+
+        if delim_search.peek().is_none() {
+            if !only_delimited {
+                out.write_all(line)?;
+                if line[line.len() - 1] != newline_char {
+                    out.write_all(&[newline_char])?;
+                }
+            }
+
+            return Ok(true);
+        }
+
+        for &Range { low, high } in ranges {
+            if low - fields_pos > 0 {
+                if let Some((first, last)) = delim_search.nth(low - fields_pos - 1) {
+                    low_idx = if print_delim { first } else { last }
+                } else {
+                    break;
+                }
+            }
+
+            match delim_search.nth(high - low) {
+                Some((first, _)) => {
+                    let segment = &line[low_idx..first];
+
+                    out.write_all(segment)?;
+
+                    print_delim = true;
+                    low_idx = first;
+                    fields_pos = high + 1;
+                }
+                None => {
+                    let segment = &line[low_idx..line.len()];
+
+                    out.write_all(segment)?;
+
+                    if line[line.len() - 1] == newline_char {
+                        return Ok(true);
+                    }
+                    break;
+                }
+            }
+        }
+        out.write_all(&[newline_char])?;
+        Ok(true)
+    });
+
+    if let Err(e) = result {
+        return Err(USimpleError::new(1, e.to_string()));
+    }
+
+    Ok(())
+}
+
 fn cut_fields<R: Read>(reader: R, ranges: &[Range], opts: &FieldOptions) -> UResult<()> {
     let newline_char = if opts.zero_terminated { b'\0' } else { b'\n' };
     match opts.delimiter {
-        Delimiter::Whitespace => cut_fields_whitespace(
-            reader,
-            ranges,
-            opts.only_delimited,
-            newline_char,
-            opts.out_delimiter.as_deref().unwrap_or("\t"),
-        ),
-        Delimiter::String(ref delimiter) => {
-            if let Some(ref o_delim) = opts.out_delimiter {
-                return cut_fields_delimiter(
+        Delimiter::String(ref delim) => {
+            let matcher = ExactMatcher::new(delim.as_bytes());
+            match opts.out_delimiter {
+                Some(ref out_delim) => cut_fields_explicit_out_delim(
                     reader,
+                    &matcher,
                     ranges,
-                    delimiter,
                     opts.only_delimited,
                     newline_char,
-                    o_delim,
-                );
+                    out_delim,
+                ),
+                None => cut_fields_implicit_out_delim(
+                    reader,
+                    &matcher,
+                    ranges,
+                    opts.only_delimited,
+                    newline_char,
+                ),
             }
-
-            let mut buf_in = BufReader::new(reader);
-            let mut out = stdout_writer();
-            let delim_len = delimiter.len();
-
-            let result = buf_in.for_byte_record_with_terminator(newline_char, |line| {
-                let mut fields_pos = 1;
-                let mut low_idx = 0;
-                let mut delim_search = Searcher::new(line, delimiter.as_bytes()).peekable();
-                let mut print_delim = false;
-
-                if delim_search.peek().is_none() {
-                    if !opts.only_delimited {
-                        out.write_all(line)?;
-                        if line[line.len() - 1] != newline_char {
-                            out.write_all(&[newline_char])?;
-                        }
-                    }
-
-                    return Ok(true);
-                }
-
-                for &Range { low, high } in ranges {
-                    if low - fields_pos > 0 {
-                        if let Some(delim_pos) = delim_search.nth(low - fields_pos - 1) {
-                            low_idx = if print_delim {
-                                delim_pos
-                            } else {
-                                delim_pos + delim_len
-                            }
-                        } else {
-                            break;
-                        }
-                    }
-
-                    match delim_search.nth(high - low) {
-                        Some(high_idx) => {
-                            let segment = &line[low_idx..high_idx];
-
-                            out.write_all(segment)?;
-
-                            print_delim = true;
-                            low_idx = high_idx;
-                            fields_pos = high + 1;
-                        }
-                        None => {
-                            let segment = &line[low_idx..line.len()];
-
-                            out.write_all(segment)?;
-
-                            if line[line.len() - 1] == newline_char {
-                                return Ok(true);
-                            }
-                            break;
-                        }
-                    }
-                }
-                out.write_all(&[newline_char])?;
-                Ok(true)
-            });
-
-            if let Err(e) = result {
-                return Err(USimpleError::new(1, e.to_string()));
-            }
-
-            Ok(())
+        }
+        Delimiter::Whitespace => {
+            let matcher = WhitespaceMatcher {};
+            let out_delim = opts.out_delimiter.as_deref().unwrap_or("\t");
+            cut_fields_explicit_out_delim(
+                reader,
+                &matcher,
+                ranges,
+                opts.only_delimited,
+                newline_char,
+                out_delim,
+            )
         }
     }
 }
@@ -656,7 +509,7 @@ pub fn uu_app() -> Command {
         .version(crate_version!())
         .override_usage(format_usage(USAGE))
         .about(ABOUT)
-        .after_help(LONG_HELP)
+        .after_help(AFTER_HELP)
         .infer_long_args(true)
         .arg(
             Arg::new(options::BYTES)
