@@ -14,20 +14,25 @@ use std::{
     time::Duration,
 };
 
-use clap::{crate_version, Arg, ArgAction, Command};
+use clap::{crate_version, value_parser, Arg, ArgAction, ArgMatches, Command};
 use crossterm::event::KeyEventKind;
 use crossterm::{
+    cursor::{MoveTo, MoveUp},
     event::{self, Event, KeyCode, KeyEvent, KeyModifiers},
     execute, queue,
     style::Attribute,
-    terminal,
+    terminal::{self, Clear, ClearType},
 };
 
+use is_terminal::IsTerminal;
 use unicode_segmentation::UnicodeSegmentation;
 use unicode_width::UnicodeWidthStr;
 use uucore::display::Quotable;
 use uucore::error::{UResult, USimpleError, UUsageError};
+use uucore::{format_usage, help_about, help_usage};
 
+const ABOUT: &str = help_about!("more.md");
+const USAGE: &str = help_usage!("more.md");
 const BELL: &str = "\x07";
 
 pub mod options {
@@ -45,14 +50,56 @@ pub mod options {
     pub const FILES: &str = "files";
 }
 
-const MULTI_FILE_TOP_PROMPT: &str = "::::::::::::::\n{}\n::::::::::::::\n";
+const MULTI_FILE_TOP_PROMPT: &str = "\r::::::::::::::\n\r{}\n\r::::::::::::::\n";
+
+struct Options {
+    clean_print: bool,
+    from_line: usize,
+    lines: Option<u16>,
+    print_over: bool,
+    silent: bool,
+    squeeze: bool,
+}
+
+impl Options {
+    fn from(matches: &ArgMatches) -> Self {
+        let lines = match (
+            matches.get_one::<u16>(options::LINES).copied(),
+            matches.get_one::<u16>(options::NUMBER).copied(),
+        ) {
+            // We add 1 to the number of lines to display because the last line
+            // is used for the banner
+            (Some(number), _) if number > 0 => Some(number + 1),
+            (None, Some(number)) if number > 0 => Some(number + 1),
+            (_, _) => None,
+        };
+        let from_line = match matches.get_one::<usize>(options::FROM_LINE).copied() {
+            Some(number) if number > 1 => number - 1,
+            _ => 0,
+        };
+        Self {
+            clean_print: matches.get_flag(options::CLEAN_PRINT),
+            from_line,
+            lines,
+            print_over: matches.get_flag(options::PRINT_OVER),
+            silent: matches.get_flag(options::SILENT),
+            squeeze: matches.get_flag(options::SQUEEZE),
+        }
+    }
+}
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().get_matches_from(args);
+    let args = args.collect_lossy();
+    let matches = match uu_app().try_get_matches_from(&args) {
+        Ok(m) => m,
+        Err(e) => return Err(e.into()),
+    };
+
+    let mut options = Options::from(&matches);
 
     let mut buff = String::new();
-    let silent = matches.get_flag(options::SILENT);
+
     if let Some(files) = matches.get_many::<String>(options::FILES) {
         let mut stdout = setup_term();
         let length = files.len();
@@ -74,19 +121,33 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                     format!("cannot open {}: No such file or directory", file.quote()),
                 ));
             }
-            if length > 1 {
-                buff.push_str(&MULTI_FILE_TOP_PROMPT.replace("{}", file.to_str().unwrap()));
-            }
-            let mut reader = BufReader::new(File::open(file).unwrap());
+            let opened_file = match File::open(file) {
+                Err(why) => {
+                    terminal::disable_raw_mode().unwrap();
+                    return Err(USimpleError::new(
+                        1,
+                        format!("cannot open {}: {}", file.quote(), why.kind()),
+                    ));
+                }
+                Ok(opened_file) => opened_file,
+            };
+            let mut reader = BufReader::new(opened_file);
             reader.read_to_string(&mut buff).unwrap();
-            more(&buff, &mut stdout, next_file.copied(), silent)?;
+            more(
+                &buff,
+                &mut stdout,
+                length > 1,
+                file.to_str(),
+                next_file.copied(),
+                &mut options,
+            )?;
             buff.clear();
         }
         reset_term(&mut stdout);
-    } else if atty::isnt(atty::Stream::Stdin) {
+    } else if !std::io::stdin().is_terminal() {
         stdin().read_to_string(&mut buff).unwrap();
         let mut stdout = setup_term();
-        more(&buff, &mut stdout, None, silent)?;
+        more(&buff, &mut stdout, false, None, None, &mut options)?;
         reset_term(&mut stdout);
     } else {
         return Err(UUsageError::new(1, "bad usage"));
@@ -96,9 +157,17 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .about("A file perusal filter for CRT viewing.")
+        .about(ABOUT)
+        .override_usage(format_usage(USAGE))
         .version(crate_version!())
         .infer_long_args(true)
+        .arg(
+            Arg::new(options::PRINT_OVER)
+                .short('c')
+                .long(options::PRINT_OVER)
+                .help("Do not scroll, display text and clean line ends")
+                .action(ArgAction::SetTrue),
+        )
         .arg(
             Arg::new(options::SILENT)
                 .short('d')
@@ -106,6 +175,52 @@ pub fn uu_app() -> Command {
                 .help("Display help instead of ringing bell")
                 .action(ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new(options::CLEAN_PRINT)
+                .short('p')
+                .long(options::CLEAN_PRINT)
+                .help("Do not scroll, clean screen and display text")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::SQUEEZE)
+                .short('s')
+                .long(options::SQUEEZE)
+                .help("Squeeze multiple blank lines into one")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::PLAIN)
+                .short('u')
+                .long(options::PLAIN)
+                .action(ArgAction::SetTrue)
+                .hide(true),
+        )
+        .arg(
+            Arg::new(options::FROM_LINE)
+                .short('F')
+                .long(options::FROM_LINE)
+                .num_args(1)
+                .value_name("number")
+                .value_parser(value_parser!(usize))
+                .help("Display file beginning from line number"),
+        )
+        .arg(
+            Arg::new(options::LINES)
+                .short('n')
+                .long(options::LINES)
+                .value_name("number")
+                .num_args(1)
+                .value_parser(value_parser!(u16).range(0..))
+                .help("The number of lines per screen full"),
+        )
+        .arg(
+            Arg::new(options::NUMBER)
+                .long(options::NUMBER)
+                .num_args(1)
+                .value_parser(value_parser!(u16).range(0..))
+                .help("Same as --lines"),
+        )
         // The commented arguments below are unimplemented:
         /*
         .arg(
@@ -120,55 +235,6 @@ pub fn uu_app() -> Command {
                 .long(options::NO_PAUSE)
                 .help("Suppress pause after form feed"),
         )
-        .arg(
-            Arg::new(options::PRINT_OVER)
-                .short('c')
-                .long(options::PRINT_OVER)
-                .help("Do not scroll, display text and clean line ends"),
-        )
-        .arg(
-            Arg::new(options::CLEAN_PRINT)
-                .short('p')
-                .long(options::CLEAN_PRINT)
-                .help("Do not scroll, clean screen and display text"),
-        )
-        .arg(
-            Arg::new(options::SQUEEZE)
-                .short('s')
-                .long(options::SQUEEZE)
-                .help("Squeeze multiple blank lines into one"),
-        )
-        .arg(
-            Arg::new(options::PLAIN)
-                .short('u')
-                .long(options::PLAIN)
-                .help("Suppress underlining and bold"),
-        )
-        .arg(
-            Arg::new(options::LINES)
-                .short('n')
-                .long(options::LINES)
-                .value_name("number")
-                .takes_value(true)
-                .help("The number of lines per screen full"),
-        )
-        .arg(
-            Arg::new(options::NUMBER)
-                .allow_hyphen_values(true)
-                .long(options::NUMBER)
-                .required(false)
-                .takes_value(true)
-                .help("Same as --lines"),
-        )
-        .arg(
-            Arg::new(options::FROM_LINE)
-                .short('F')
-                .allow_hyphen_values(true)
-                .required(false)
-                .takes_value(true)
-                .value_name("number")
-                .help("Display file beginning from line number"),
-        )
         .arg(
             Arg::new(options::PATTERN)
                 .short('P')
@@ -204,7 +270,7 @@ fn setup_term() -> usize {
 fn reset_term(stdout: &mut std::io::Stdout) {
     terminal::disable_raw_mode().unwrap();
     // Clear the prompt
-    queue!(stdout, terminal::Clear(terminal::ClearType::CurrentLine)).unwrap();
+    queue!(stdout, terminal::Clear(ClearType::CurrentLine)).unwrap();
     // Move cursor to the beginning without printing new line
     print!("\r");
     stdout.flush().unwrap();
@@ -214,13 +280,39 @@ fn reset_term(stdout: &mut std::io::Stdout) {
 #[inline(always)]
 fn reset_term(_: &mut usize) {}
 
-fn more(buff: &str, stdout: &mut Stdout, next_file: Option<&str>, silent: bool) -> UResult<()> {
-    let (cols, rows) = terminal::size().unwrap();
+fn more(
+    buff: &str,
+    stdout: &mut Stdout,
+    multiple_file: bool,
+    file: Option<&str>,
+    next_file: Option<&str>,
+    options: &mut Options,
+) -> UResult<()> {
+    let (cols, mut rows) = terminal::size().unwrap();
+    if let Some(number) = options.lines {
+        rows = number;
+    }
+
     let lines = break_buff(buff, usize::from(cols));
 
-    let mut pager = Pager::new(rows, lines, next_file, silent);
+    let mut pager = Pager::new(rows, lines, next_file, options);
+
+    if multiple_file {
+        execute!(stdout, terminal::Clear(terminal::ClearType::CurrentLine)).unwrap();
+        stdout.write_all(
+            MULTI_FILE_TOP_PROMPT
+                .replace("{}", file.unwrap_or_default())
+                .as_bytes(),
+        )?;
+        pager.content_rows -= 3;
+    }
     pager.draw(stdout, None);
-    if pager.should_close() {
+    if multiple_file {
+        options.from_line = 0;
+        pager.content_rows += 3;
+    }
+
+    if pager.should_close() && next_file.is_none() {
         return Ok(());
     }
 
@@ -252,6 +344,11 @@ fn more(buff: &str, stdout: &mut Stdout, next_file: Option<&str>, silent: bool)
                     modifiers: KeyModifiers::NONE,
                     ..
                 })
+                | Event::Key(KeyEvent {
+                    code: KeyCode::PageDown,
+                    modifiers: KeyModifiers::NONE,
+                    ..
+                })
                 | Event::Key(KeyEvent {
                     code: KeyCode::Char(' '),
                     modifiers: KeyModifiers::NONE,
@@ -267,8 +364,14 @@ fn more(buff: &str, stdout: &mut Stdout, next_file: Option<&str>, silent: bool)
                     code: KeyCode::Up,
                     modifiers: KeyModifiers::NONE,
                     ..
+                })
+                | Event::Key(KeyEvent {
+                    code: KeyCode::PageUp,
+                    modifiers: KeyModifiers::NONE,
+                    ..
                 }) => {
                     pager.page_up();
+                    paging_add_back_message(options, stdout)?;
                 }
                 Event::Key(KeyEvent {
                     code: KeyCode::Char('j'),
@@ -289,7 +392,7 @@ fn more(buff: &str, stdout: &mut Stdout, next_file: Option<&str>, silent: bool)
                     pager.prev_line();
                 }
                 Event::Resize(col, row) => {
-                    pager.page_resize(col, row);
+                    pager.page_resize(col, row, options.lines);
                 }
                 Event::Key(KeyEvent {
                     code: KeyCode::Char(k),
@@ -298,6 +401,16 @@ fn more(buff: &str, stdout: &mut Stdout, next_file: Option<&str>, silent: bool)
                 _ => continue,
             }
 
+            if options.print_over {
+                execute!(
+                    std::io::stdout(),
+                    MoveTo(0, 0),
+                    Clear(ClearType::FromCursorDown)
+                )
+                .unwrap();
+            } else if options.clean_print {
+                execute!(std::io::stdout(), Clear(ClearType::All), MoveTo(0, 0)).unwrap();
+            }
             pager.draw(stdout, wrong_key);
         }
     }
@@ -312,18 +425,22 @@ struct Pager<'a> {
     next_file: Option<&'a str>,
     line_count: usize,
     silent: bool,
+    squeeze: bool,
+    line_squeezed: usize,
 }
 
 impl<'a> Pager<'a> {
-    fn new(rows: u16, lines: Vec<String>, next_file: Option<&'a str>, silent: bool) -> Self {
+    fn new(rows: u16, lines: Vec<String>, next_file: Option<&'a str>, options: &Options) -> Self {
         let line_count = lines.len();
         Self {
-            upper_mark: 0,
+            upper_mark: options.from_line,
             content_rows: rows.saturating_sub(1),
             lines,
             next_file,
             line_count,
-            silent,
+            silent: options.silent,
+            squeeze: options.squeeze,
+            line_squeezed: 0,
         }
     }
 
@@ -349,7 +466,21 @@ impl<'a> Pager<'a> {
     }
 
     fn page_up(&mut self) {
-        self.upper_mark = self.upper_mark.saturating_sub(self.content_rows.into());
+        let content_row_usize: usize = self.content_rows.into();
+        self.upper_mark = self
+            .upper_mark
+            .saturating_sub(content_row_usize.saturating_add(self.line_squeezed));
+
+        if self.squeeze {
+            let iter = self.lines.iter().take(self.upper_mark).rev();
+            for line in iter {
+                if line.is_empty() {
+                    self.upper_mark = self.upper_mark.saturating_sub(1);
+                } else {
+                    break;
+                }
+            }
+        }
     }
 
     fn next_line(&mut self) {
@@ -361,31 +492,62 @@ impl<'a> Pager<'a> {
     }
 
     // TODO: Deal with column size changes.
-    fn page_resize(&mut self, _: u16, row: u16) {
-        self.content_rows = row.saturating_sub(1);
+    fn page_resize(&mut self, _: u16, row: u16, option_line: Option<u16>) {
+        if option_line.is_none() {
+            self.content_rows = row.saturating_sub(1);
+        };
     }
 
-    fn draw(&self, stdout: &mut std::io::Stdout, wrong_key: Option<char>) {
+    fn draw(&mut self, stdout: &mut std::io::Stdout, wrong_key: Option<char>) {
+        self.draw_lines(stdout);
         let lower_mark = self
             .line_count
             .min(self.upper_mark.saturating_add(self.content_rows.into()));
-        self.draw_lines(stdout);
         self.draw_prompt(stdout, lower_mark, wrong_key);
         stdout.flush().unwrap();
     }
 
-    fn draw_lines(&self, stdout: &mut std::io::Stdout) {
+    fn draw_lines(&mut self, stdout: &mut std::io::Stdout) {
         execute!(stdout, terminal::Clear(terminal::ClearType::CurrentLine)).unwrap();
-        let displayed_lines = self
-            .lines
-            .iter()
-            .skip(self.upper_mark)
-            .take(self.content_rows.into());
+
+        self.line_squeezed = 0;
+        let mut previous_line_blank = false;
+        let mut displayed_lines = Vec::new();
+        let mut iter = self.lines.iter().skip(self.upper_mark);
+
+        while displayed_lines.len() < self.content_rows as usize {
+            match iter.next() {
+                Some(line) => {
+                    if self.squeeze {
+                        match (line.is_empty(), previous_line_blank) {
+                            (true, false) => {
+                                previous_line_blank = true;
+                                displayed_lines.push(line);
+                            }
+                            (false, true) => {
+                                previous_line_blank = false;
+                                displayed_lines.push(line);
+                            }
+                            (false, false) => displayed_lines.push(line),
+                            (true, true) => {
+                                self.line_squeezed += 1;
+                                self.upper_mark += 1;
+                            }
+                        }
+                    } else {
+                        displayed_lines.push(line);
+                    }
+                }
+                // if none the end of the file is reached
+                None => {
+                    self.upper_mark = self.line_count;
+                    break;
+                }
+            }
+        }
 
         for line in displayed_lines {
-            stdout
-                .write_all(format!("\r{}\n", line).as_bytes())
-                .unwrap();
+            stdout.write_all(format!("\r{line}\n").as_bytes()).unwrap();
         }
     }
 
@@ -399,15 +561,13 @@ impl<'a> Pager<'a> {
             )
         };
 
-        let status = format!("--More--({})", status_inner);
-
+        let status = format!("--More--({status_inner})");
         let banner = match (self.silent, wrong_key) {
             (true, Some(key)) => format!(
-                "{} [Unknown key: '{}'. Press 'h' for instructions. (unimplemented)]",
-                status, key
+                "{status} [Unknown key: '{key}'. Press 'h' for instructions. (unimplemented)]"
             ),
-            (true, None) => format!("{}[Press space to continue, 'q' to quit.]", status),
-            (false, Some(_)) => format!("{}{}", status, BELL),
+            (true, None) => format!("{status}[Press space to continue, 'q' to quit.]"),
+            (false, Some(_)) => format!("{status}{BELL}"),
             (false, None) => status,
         };
 
@@ -422,6 +582,14 @@ impl<'a> Pager<'a> {
     }
 }
 
+fn paging_add_back_message(options: &Options, stdout: &mut std::io::Stdout) -> UResult<()> {
+    if options.lines.is_some() {
+        execute!(stdout, MoveUp(1))?;
+        stdout.write_all("\n\r...back 1 page\n".as_bytes())?;
+    }
+    Ok(())
+}
+
 // Break the lines on the cols of the terminal
 fn break_buff(buff: &str, cols: usize) -> Vec<String> {
     let mut lines = Vec::with_capacity(buff.lines().count());
