@@ -14,11 +14,11 @@ use std::path::Path;
 use clap::builder::ValueParser;
 use clap::{crate_version, Arg, Command};
 use uucore::error::UResult;
-use uucore::format_usage;
 use uucore::utmpx::{self, Utmpx};
+use uucore::{format_usage, help_about, help_usage};
 
-static ABOUT: &str = "Print the user names of users currently logged in to the current host";
-const USAGE: &str = "{} [FILE]";
+const ABOUT: &str = help_about!("users.md");
+const USAGE: &str = help_usage!("users.md");
 
 static ARG_FILES: &str = "files";
 
@@ -41,10 +41,10 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         .map(|v| v.map(AsRef::as_ref).collect())
         .unwrap_or_default();
 
-    let filename = if !files.is_empty() {
-        files[0]
-    } else {
+    let filename = if files.is_empty() {
         utmpx::DEFAULT_FILE.as_ref()
+    } else {
+        files[0]
     };
 
     let mut users = Utmpx::iter_all_records_from(filename)
