@@ -396,7 +396,7 @@ impl Display for UIoError {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), std::fmt::Error> {
         use std::io::ErrorKind::*;
 
-        let mut message;
+        let message;
         let message = if self.inner.raw_os_error().is_some() {
             // These are errors that come directly from the OS.
             // We want to normalize their messages across systems,
@@ -424,7 +424,6 @@ impl Display for UIoError {
                     // (https://github.com/rust-lang/rust/issues/86442)
                     // are stabilized, we should add them to the match statement.
                     message = strip_errno(&self.inner);
-                    capitalize(&mut message);
                     &message
                 }
             }
@@ -435,24 +434,16 @@ impl Display for UIoError {
             // a file that was not found.
             // There are also errors with entirely custom messages.
             message = self.inner.to_string();
-            capitalize(&mut message);
             &message
         };
         if let Some(ctx) = &self.context {
-            write!(f, "{}: {}", ctx, message)
+            write!(f, "{ctx}: {message}")
         } else {
-            write!(f, "{}", message)
+            write!(f, "{message}")
         }
     }
 }
 
-/// Capitalize the first character of an ASCII string.
-fn capitalize(text: &mut str) {
-    if let Some(first) = text.get_mut(..1) {
-        first.make_ascii_uppercase();
-    }
-}
-
 /// Strip the trailing " (os error XX)" from io error strings.
 pub fn strip_errno(err: &std::io::Error) -> String {
     let mut msg = err.to_string();
@@ -522,7 +513,7 @@ impl From<std::io::Error> for Box<dyn UError> {
 /// // prints "fix me please!: Permission denied"
 /// println!("{}", uio_result.unwrap_err());
 /// ```
-#[cfg(any(target_os = "linux", target_os = "android"))]
+#[cfg(unix)]
 impl<T> FromIo<UResult<T>> for Result<T, nix::Error> {
     fn map_err_context(self, context: impl FnOnce() -> String) -> UResult<T> {
         self.map_err(|e| {
@@ -534,7 +525,7 @@ impl<T> FromIo<UResult<T>> for Result<T, nix::Error> {
     }
 }
 
-#[cfg(any(target_os = "linux", target_os = "android"))]
+#[cfg(unix)]
 impl<T> FromIo<UResult<T>> for nix::Error {
     fn map_err_context(self, context: impl FnOnce() -> String) -> UResult<T> {
         Err(Box::new(UIoError {
@@ -544,7 +535,7 @@ impl<T> FromIo<UResult<T>> for nix::Error {
     }
 }
 
-#[cfg(any(target_os = "linux", target_os = "android"))]
+#[cfg(unix)]
 impl From<nix::Error> for UIoError {
     fn from(f: nix::Error) -> Self {
         Self {
@@ -554,7 +545,7 @@ impl From<nix::Error> for UIoError {
     }
 }
 
-#[cfg(any(target_os = "linux", target_os = "android"))]
+#[cfg(unix)]
 impl From<nix::Error> for Box<dyn UError> {
     fn from(f: nix::Error) -> Self {
         let u_error: UIoError = f.into();
@@ -751,7 +742,7 @@ impl Display for ClapErrorWrapper {
 #[cfg(test)]
 mod tests {
     #[test]
-    #[cfg(any(target_os = "linux", target_os = "android"))]
+    #[cfg(unix)]
     fn test_nix_error_conversion() {
         use super::{FromIo, UIoError};
         use nix::errno::Errno;
@@ -771,6 +762,6 @@ mod tests {
                 .map_err_context(|| String::from("test"))
                 .unwrap_err()
                 .to_string()
-        )
+        );
     }
 }
