@@ -1,4 +1,4 @@
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 #[test]
 fn test_z85_not_padded() {
@@ -7,10 +7,10 @@ fn test_z85_not_padded() {
         .args(&["--z85", "-d"])
         .pipe_in("##########")
         .fails()
-        .stderr_only("basenc: error: invalid input");
+        .stderr_only("basenc: error: invalid input\n");
     new_ucmd!()
         .args(&["--z85"])
         .pipe_in("123")
         .fails()
-        .stderr_only("basenc: error: invalid input (length must be multiple of 4 characters)");
+        .stderr_only("basenc: error: invalid input (length must be multiple of 4 characters)\n");
 }
