@@ -1,10 +1,8 @@
 // spell-checker:ignore (words) agroupthatdoesntexist auserthatdoesntexist cuuser groupname notexisting passgrp
 
-use crate::common::util::*;
+use crate::common::util::{is_ci, run_ucmd_as_root, CmdResult, TestScenario};
 #[cfg(any(target_os = "linux", target_os = "android"))]
-use rust_users::get_effective_uid;
-
-extern crate chown;
+use uucore::process::geteuid;
 
 // Apparently some CI environments have configuration issues, e.g. with 'whoami' and 'id'.
 // If we are running inside the CI and "needle" is in "stderr" skipping this test is
@@ -36,7 +34,7 @@ fn skipping_test_is_okay(result: &CmdResult, needle: &str) -> bool {
 
 #[cfg(test)]
 mod test_passgrp {
-    use super::chown::entries::{gid2grp, grp2gid, uid2usr, usr2uid};
+    use chown::entries::{gid2grp, grp2gid, uid2usr, usr2uid};
 
     #[test]
     fn test_usr2uid() {
@@ -138,7 +136,7 @@ fn test_chown_only_owner_colon() {
 
     scene
         .ucmd()
-        .arg(format!("{}:", user_name))
+        .arg(format!("{user_name}:"))
         .arg("--verbose")
         .arg(file1)
         .succeeds()
@@ -146,7 +144,7 @@ fn test_chown_only_owner_colon() {
 
     scene
         .ucmd()
-        .arg(format!("{}.", user_name))
+        .arg(format!("{user_name}."))
         .arg("--verbose")
         .arg(file1)
         .succeeds()
@@ -244,7 +242,7 @@ fn test_chown_owner_group() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_name, group_name))
+        .arg(format!("{user_name}:{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -309,7 +307,7 @@ fn test_chown_various_input() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_name, group_name))
+        .arg(format!("{user_name}:{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -321,7 +319,7 @@ fn test_chown_various_input() {
     // check that username.groupname is understood
     let result = scene
         .ucmd()
-        .arg(format!("{}.{}", user_name, group_name))
+        .arg(format!("{user_name}.{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -365,7 +363,7 @@ fn test_chown_only_group() {
 
     let result = scene
         .ucmd()
-        .arg(format!(":{}", user_name))
+        .arg(format!(":{user_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -396,7 +394,7 @@ fn test_chown_only_user_id() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-u").run();
+    let result = scene.cmd("id").arg("-u").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -430,7 +428,7 @@ fn test_chown_fail_id() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-u").run();
+    let result = scene.cmd("id").arg("-u").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -442,14 +440,14 @@ fn test_chown_fail_id() {
 
     scene
         .ucmd()
-        .arg(format!("{}:", user_id))
+        .arg(format!("{user_id}:"))
         .arg(file1)
         .fails()
         .stderr_contains("invalid spec");
 
     scene
         .ucmd()
-        .arg(format!("{}.", user_id))
+        .arg(format!("{user_id}."))
         .arg(file1)
         .fails()
         .stderr_contains("invalid spec");
@@ -487,7 +485,7 @@ fn test_chown_only_group_id() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-g").run();
+    let result = scene.cmd("id").arg("-g").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -499,7 +497,7 @@ fn test_chown_only_group_id() {
 
     let result = scene
         .ucmd()
-        .arg(format!(":{}", group_id))
+        .arg(format!(":{group_id}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -551,14 +549,14 @@ fn test_chown_owner_group_id() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-u").run();
+    let result = scene.cmd("id").arg("-u").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
     let user_id = String::from(result.stdout_str().trim());
     assert!(!user_id.is_empty());
 
-    let result = scene.cmd_keepenv("id").arg("-g").run();
+    let result = scene.cmd("id").arg("-g").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -570,7 +568,7 @@ fn test_chown_owner_group_id() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_id, group_id))
+        .arg(format!("{user_id}:{group_id}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -583,7 +581,7 @@ fn test_chown_owner_group_id() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}.{}", user_id, group_id))
+        .arg(format!("{user_id}.{group_id}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -612,14 +610,14 @@ fn test_chown_owner_group_mix() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-u").run();
+    let result = scene.cmd("id").arg("-u").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
     let user_id = String::from(result.stdout_str().trim());
     assert!(!user_id.is_empty());
 
-    let result = scene.cmd_keepenv("id").arg("-gn").run();
+    let result = scene.cmd("id").arg("-gn").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -631,7 +629,7 @@ fn test_chown_owner_group_mix() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_id, group_name))
+        .arg(format!("{user_id}:{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -662,10 +660,10 @@ fn test_chown_recursive() {
 
     at.mkdir_all("a/b/c");
     at.mkdir("z");
-    at.touch(&at.plus_as_string("a/a"));
-    at.touch(&at.plus_as_string("a/b/b"));
-    at.touch(&at.plus_as_string("a/b/c/c"));
-    at.touch(&at.plus_as_string("z/y"));
+    at.touch(at.plus_as_string("a/a"));
+    at.touch(at.plus_as_string("a/b/b"));
+    at.touch(at.plus_as_string("a/b/c/c"));
+    at.touch(at.plus_as_string("z/y"));
 
     let result = scene
         .ucmd()
@@ -703,7 +701,7 @@ fn test_root_preserve() {
 #[cfg(any(target_os = "linux", target_os = "android"))]
 #[test]
 fn test_big_p() {
-    if get_effective_uid() != 0 {
+    if geteuid() != 0 {
         new_ucmd!()
             .arg("-RP")
             .arg("bin")
@@ -732,15 +730,275 @@ fn test_chown_file_notexisting() {
     let user_name = String::from(result.stdout_str().trim());
     assert!(!user_name.is_empty());
 
-    let _result = scene
+    scene
         .ucmd()
-        .arg(user_name)
+        .arg(&user_name)
         .arg("--verbose")
         .arg("not_existing")
-        .fails();
-
-    // TODO: uncomment once "failed to change ownership of '{}' to {}" added to stdout
-    // result.stderr_contains("retained as");
+        .fails()
+        .stdout_contains(format!(
+            "failed to change ownership of 'not_existing' to {user_name}"
+        ));
     // TODO: uncomment once message changed from "cannot dereference" to "cannot access"
     // result.stderr_contains("cannot access 'not_existing': No such file or directory");
 }
+
+#[test]
+fn test_chown_no_change_to_user_from_user() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=42")
+        .arg("43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!("ownership of '{file}' retained as {user_name}\n"));
+}
+
+#[test]
+fn test_chown_no_change_to_user_from_group() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=:42")
+        .arg("43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!("ownership of '{file}' retained as {user_name}\n"));
+}
+
+#[test]
+fn test_chown_no_change_to_user_from_user_group() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=42:42")
+        .arg("43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!("ownership of '{file}' retained as {user_name}\n"));
+}
+
+#[test]
+fn test_chown_no_change_to_group_from_user() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+
+    let result = scene.cmd("id").arg("-ng").run();
+    if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
+        return;
+    }
+    let group_name = String::from(result.stdout_str().trim());
+    assert!(!group_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=42")
+        .arg(":43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!("ownership of '{file}' retained as {group_name}\n"));
+}
+
+#[test]
+fn test_chown_no_change_to_group_from_group() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+    let result = scene.cmd("id").arg("-ng").run();
+    if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
+        return;
+    }
+    let group_name = String::from(result.stdout_str().trim());
+    assert!(!group_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=:42")
+        .arg(":43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!("ownership of '{file}' retained as {group_name}\n"));
+}
+
+#[test]
+fn test_chown_no_change_to_group_from_user_group() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+    let result = scene.cmd("id").arg("-ng").run();
+    if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
+        return;
+    }
+    let group_name = String::from(result.stdout_str().trim());
+    assert!(!group_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=42:42")
+        .arg(":43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!("ownership of '{file}' retained as {group_name}\n"));
+}
+
+#[test]
+fn test_chown_no_change_to_user_group_from_user() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+
+    let result = scene.cmd("id").arg("-ng").run();
+    if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
+        return;
+    }
+    let group_name = String::from(result.stdout_str().trim());
+    assert!(!group_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=42")
+        .arg("43:43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!(
+            "ownership of '{file}' retained as {user_name}:{group_name}\n"
+        ));
+}
+
+#[test]
+fn test_chown_no_change_to_user_group_from_group() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+    let result = scene.cmd("id").arg("-ng").run();
+    if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
+        return;
+    }
+    let group_name = String::from(result.stdout_str().trim());
+    assert!(!group_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=:42")
+        .arg("43:43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!(
+            "ownership of '{file}' retained as {user_name}:{group_name}\n"
+        ));
+}
+
+#[test]
+fn test_chown_no_change_to_user_group_from_user_group() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("whoami").run();
+    if skipping_test_is_okay(&result, "whoami: cannot find name for user ID") {
+        return;
+    }
+    let user_name = String::from(result.stdout_str().trim());
+    assert!(!user_name.is_empty());
+    let result = scene.cmd("id").arg("-ng").run();
+    if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
+        return;
+    }
+    let group_name = String::from(result.stdout_str().trim());
+    assert!(!group_name.is_empty());
+
+    let file = "f";
+    at.touch(file);
+    scene
+        .ucmd()
+        .arg("-v")
+        .arg("--from=42:42")
+        .arg("43:43")
+        .arg(file)
+        .succeeds()
+        .stdout_only(format!(
+            "ownership of '{file}' retained as {user_name}:{group_name}\n"
+        ));
+}
