@@ -1,6 +1,6 @@
 use std::process::Stdio;
 
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 #[test]
 fn test_invalid_arg() {
@@ -25,8 +25,7 @@ fn test_rm_failed() {
     let file = "test_rm_one_file"; // Doesn't exist
 
     ucmd.arg(file).fails().stderr_contains(&format!(
-        "cannot remove '{}': No such file or directory",
-        file
+        "cannot remove '{file}': No such file or directory"
     ));
 }
 
@@ -55,6 +54,8 @@ fn test_rm_interactive() {
 
     at.touch(file_a);
     at.touch(file_b);
+    assert!(at.file_exists(file_a));
+    assert!(at.file_exists(file_b));
 
     scene
         .ucmd()
@@ -85,6 +86,11 @@ fn test_rm_force() {
     let file_a = "test_rm_force_a";
     let file_b = "test_rm_force_b";
 
+    at.touch(file_a);
+    at.touch(file_b);
+    assert!(at.file_exists(file_a));
+    assert!(at.file_exists(file_b));
+
     ucmd.arg("-f")
         .arg(file_a)
         .arg(file_b)
@@ -101,6 +107,11 @@ fn test_rm_force_multiple() {
     let file_a = "test_rm_force_a";
     let file_b = "test_rm_force_b";
 
+    at.touch(file_a);
+    at.touch(file_b);
+    assert!(at.file_exists(file_a));
+    assert!(at.file_exists(file_b));
+
     ucmd.arg("-f")
         .arg("-f")
         .arg("-f")
@@ -136,7 +147,7 @@ fn test_rm_empty_directory_verbose() {
         .arg("-v")
         .arg(dir)
         .succeeds()
-        .stdout_only(format!("removed directory '{}'\n", dir));
+        .stdout_only(format!("removed directory '{dir}'\n"));
 
     assert!(!at.dir_exists(dir));
 }
@@ -145,7 +156,7 @@ fn test_rm_empty_directory_verbose() {
 fn test_rm_non_empty_directory() {
     let (at, mut ucmd) = at_and_ucmd!();
     let dir = "test_rm_non_empty_dir";
-    let file_a = &format!("{}/test_rm_non_empty_file_a", dir);
+    let file_a = &format!("{dir}/test_rm_non_empty_file_a");
 
     at.mkdir(dir);
     at.touch(file_a);
@@ -153,7 +164,7 @@ fn test_rm_non_empty_directory() {
     ucmd.arg("-d")
         .arg(dir)
         .fails()
-        .stderr_contains(&format!("cannot remove '{}': Directory not empty", dir));
+        .stderr_contains(&format!("cannot remove '{dir}': Directory not empty"));
     assert!(at.file_exists(file_a));
     assert!(at.dir_exists(dir));
 }
@@ -208,7 +219,7 @@ fn test_rm_directory_without_flag() {
 
     ucmd.arg(dir)
         .fails()
-        .stderr_contains(&format!("cannot remove '{}': Is a directory", dir));
+        .stderr_contains(&format!("cannot remove '{dir}': Is a directory"));
 }
 
 #[test]
@@ -224,7 +235,7 @@ fn test_rm_verbose() {
         .arg(file_a)
         .arg(file_b)
         .succeeds()
-        .stdout_only(format!("removed '{}'\nremoved '{}'\n", file_a, file_b));
+        .stdout_only(format!("removed '{file_a}'\nremoved '{file_b}'\n"));
 }
 
 #[test]
@@ -259,7 +270,7 @@ fn test_rm_symlink_dir() {
         .ucmd()
         .arg(link)
         .fails()
-        .stderr_contains(&format!("cannot remove '{}': Is a directory", link));
+        .stderr_contains(&format!("cannot remove '{link}': Is a directory"));
 
     assert!(at.dir_exists(link));
 
@@ -293,7 +304,7 @@ fn test_rm_no_operand() {
 fn test_rm_verbose_slash() {
     let (at, mut ucmd) = at_and_ucmd!();
     let dir = "test_rm_verbose_slash_directory";
-    let file_a = &format!("{}/test_rm_verbose_slash_file_a", dir);
+    let file_a = &format!("{dir}/test_rm_verbose_slash_file_a");
 
     at.mkdir(dir);
     at.touch(file_a);
@@ -307,11 +318,10 @@ fn test_rm_verbose_slash() {
     ucmd.arg("-r")
         .arg("-f")
         .arg("-v")
-        .arg(&format!("{}///", dir))
+        .arg(&format!("{dir}///"))
         .succeeds()
         .stdout_only(format!(
-            "removed '{}'\nremoved directory '{}'\n",
-            file_a_normalized, dir
+            "removed '{file_a_normalized}'\nremoved directory '{dir}'\n"
         ));
 
     assert!(!at.dir_exists(dir));
@@ -351,6 +361,74 @@ fn test_rm_interactive_never() {
     assert!(!at.file_exists(file_2));
 }
 
+#[test]
+fn test_rm_interactive_missing_value() {
+    // `--interactive` is equivalent to `--interactive=always` or `-i`
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let file1 = "test_rm_interactive_missing_value_file1";
+    let file2 = "test_rm_interactive_missing_value_file2";
+
+    at.touch(file1);
+    at.touch(file2);
+
+    ucmd.arg("--interactive")
+        .arg(file1)
+        .arg(file2)
+        .pipe_in("y\ny")
+        .succeeds();
+
+    assert!(!at.file_exists(file1));
+    assert!(!at.file_exists(file2));
+}
+
+#[test]
+fn test_rm_interactive_once_prompt() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let file1 = "test_rm_interactive_once_recursive_prompt_file1";
+    let file2 = "test_rm_interactive_once_recursive_prompt_file2";
+    let file3 = "test_rm_interactive_once_recursive_prompt_file3";
+    let file4 = "test_rm_interactive_once_recursive_prompt_file4";
+
+    at.touch(file1);
+    at.touch(file2);
+    at.touch(file3);
+    at.touch(file4);
+
+    ucmd.arg("--interactive=once")
+        .arg(file1)
+        .arg(file2)
+        .arg(file3)
+        .arg(file4)
+        .pipe_in("y")
+        .succeeds()
+        .stderr_contains("remove 4 arguments?");
+
+    assert!(!at.file_exists(file1));
+    assert!(!at.file_exists(file2));
+    assert!(!at.file_exists(file3));
+    assert!(!at.file_exists(file4));
+}
+
+#[test]
+fn test_rm_interactive_once_recursive_prompt() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let file1 = "test_rm_interactive_once_recursive_prompt_file1";
+
+    at.touch(file1);
+
+    ucmd.arg("--interactive=once")
+        .arg("-r")
+        .arg(file1)
+        .pipe_in("y")
+        .succeeds()
+        .stderr_contains("remove 1 argument recursively?");
+
+    assert!(!at.file_exists(file1));
+}
+
 #[test]
 fn test_rm_descend_directory() {
     // This test descends into each directory and deletes the files and folders inside of them
@@ -359,7 +437,7 @@ fn test_rm_descend_directory() {
     // Needed for talking with stdin on platforms where CRLF or LF matters
     const END_OF_LINE: &str = if cfg!(windows) { "\r\n" } else { "\n" };
 
-    let yes = format!("y{}", END_OF_LINE);
+    let yes = format!("y{END_OF_LINE}");
 
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -414,7 +492,7 @@ fn test_rm_prompts() {
 
     answers.sort();
 
-    let yes = format!("y{}", END_OF_LINE);
+    let yes = format!("y{END_OF_LINE}");
 
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -463,7 +541,7 @@ fn test_rm_prompts() {
     let mut trimmed_output = Vec::new();
     for string in result.stderr_str().split("rm: ") {
         if !string.is_empty() {
-            let trimmed_string = format!("rm: {}", string).trim().to_string();
+            let trimmed_string = format!("rm: {string}").trim().to_string();
             trimmed_output.push(trimmed_string);
         }
     }
@@ -484,7 +562,7 @@ fn test_rm_force_prompts_order() {
     // Needed for talking with stdin on platforms where CRLF or LF matters
     const END_OF_LINE: &str = if cfg!(windows) { "\r\n" } else { "\n" };
 
-    let yes = format!("y{}", END_OF_LINE);
+    let yes = format!("y{END_OF_LINE}");
 
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -503,11 +581,8 @@ fn test_rm_force_prompts_order() {
     child.try_write_in(yes.as_bytes()).unwrap();
 
     let result = child.wait().unwrap();
-    let string_output = result.stderr_str();
-    assert_eq!(
-        string_output.trim(),
-        "rm: remove regular empty file 'empty'?"
-    );
+    result.stderr_only("rm: remove regular empty file 'empty'? ");
+
     assert!(!at.file_exists(empty_file));
 
     at.touch(empty_file);
@@ -583,3 +658,24 @@ fn test_fifo_removal() {
         .timeout(Duration::from_secs(2))
         .succeeds();
 }
+
+#[test]
+#[cfg(any(unix, target_os = "wasi"))]
+#[cfg(not(target_os = "macos"))]
+fn test_non_utf8() {
+    use std::ffi::OsStr;
+    #[cfg(unix)]
+    use std::os::unix::ffi::OsStrExt;
+    #[cfg(target_os = "wasi")]
+    use std::os::wasi::ffi::OsStrExt;
+
+    let file = OsStr::from_bytes(b"not\xffutf8"); // spell-checker:disable-line
+
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.touch(file);
+    assert!(at.file_exists(file));
+
+    ucmd.arg(file).succeeds();
+    assert!(!at.file_exists(file));
+}
