@@ -1,5 +1,5 @@
 // spell-checker:ignore lmnop xlmnop
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 use std::process::Stdio;
 
 #[test]
@@ -7,6 +7,14 @@ fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
 }
 
+#[test]
+fn test_no_args() {
+    new_ucmd!()
+        .fails()
+        .code_is(1)
+        .stderr_contains("missing operand");
+}
+
 #[test]
 fn test_hex_rejects_sign_after_identifier() {
     new_ucmd!()
@@ -36,18 +44,12 @@ fn test_hex_rejects_sign_after_identifier() {
         .args(&["-0x-123ABC"])
         .fails()
         .no_stdout()
-        .stderr_contains(
-            "Found argument '-0' which wasn't expected, or isn't valid in this context",
-        )
-        .stderr_contains("For more information try '--help'");
+        .stderr_contains("unexpected argument '-0' found");
     new_ucmd!()
         .args(&["-0x+123ABC"])
         .fails()
         .no_stdout()
-        .stderr_contains(
-            "Found argument '-0' which wasn't expected, or isn't valid in this context",
-        )
-        .stderr_contains("For more information try '--help'");
+        .stderr_contains("unexpected argument '-0' found");
 }
 
 #[test]
@@ -214,10 +216,13 @@ fn test_separator_and_terminator() {
 
 #[test]
 fn test_equalize_widths() {
-    new_ucmd!()
-        .args(&["-w", "5", "10"])
-        .run()
-        .stdout_is("05\n06\n07\n08\n09\n10\n");
+    let args = ["-w", "--equal-width"];
+    for arg in args {
+        new_ucmd!()
+            .args(&[arg, "5", "10"])
+            .run()
+            .stdout_is("05\n06\n07\n08\n09\n10\n");
+    }
 }
 
 #[test]
