@@ -10,7 +10,7 @@
 
 // spell-checker:ignore (words) egid euid pseudofloat
 
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 use std::thread::sleep;
 
 #[test]
@@ -288,7 +288,7 @@ fn test_float_inequality_is_error() {
         .args(&["123.45", "-ge", "6"])
         .run()
         .code_is(2)
-        .stderr_is("test: invalid integer '123.45'");
+        .stderr_is("test: invalid integer '123.45'\n");
 }
 
 #[test]
@@ -300,21 +300,17 @@ fn test_invalid_utf8_integer_compare() {
     let source = [0x66, 0x6f, 0x80, 0x6f];
     let arg = OsStr::from_bytes(&source[..]);
 
-    let mut cmd = new_ucmd!();
-    cmd.arg("123").arg("-ne");
-    cmd.raw.arg(arg);
-
-    cmd.run()
+    new_ucmd!()
+        .args(&[OsStr::new("123"), OsStr::new("-ne"), arg])
+        .run()
         .code_is(2)
-        .stderr_is("test: invalid integer $'fo\\x80o'");
+        .stderr_is("test: invalid integer $'fo\\x80o'\n");
 
-    let mut cmd = new_ucmd!();
-    cmd.raw.arg(arg);
-    cmd.arg("-eq").arg("456");
-
-    cmd.run()
+    new_ucmd!()
+        .args(&[arg, OsStr::new("-eq"), OsStr::new("456")])
+        .run()
         .code_is(2)
-        .stderr_is("test: invalid integer $'fo\\x80o'");
+        .stderr_is("test: invalid integer $'fo\\x80o'\n");
 }
 
 #[test]
@@ -827,7 +823,7 @@ fn test_erroneous_parenthesized_expression() {
         .args(&["a", "!=", "(", "b", "-a", "b", ")", "!=", "c"])
         .run()
         .code_is(2)
-        .stderr_is("test: extra argument 'b'");
+        .stderr_is("test: extra argument 'b'\n");
 }
 
 #[test]
@@ -835,6 +831,15 @@ fn test_or_as_filename() {
     new_ucmd!().args(&["x", "-a", "-z", "-o"]).run().code_is(1);
 }
 
+#[test]
+#[ignore = "TODO: Busybox has this working"]
+fn test_filename_or_with_equal() {
+    new_ucmd!()
+        .args(&["-f", "=", "a", "-o", "b"])
+        .run()
+        .code_is(0);
+}
+
 #[test]
 #[ignore = "GNU considers this an error"]
 fn test_string_length_and_nothing() {
@@ -866,7 +871,7 @@ fn test_bracket_syntax_missing_right_bracket() {
     ucmd.args(&["1", "-eq"])
         .run()
         .code_is(2)
-        .stderr_is("[: missing ']'");
+        .stderr_is("[: missing ']'\n");
 }
 
 #[test]
@@ -923,3 +928,16 @@ fn test_long_integer() {
         ])
         .fails();
 }
+
+#[test]
+fn test_missing_argument_after() {
+    let mut ucmd = new_ucmd!();
+
+    let result = ucmd.args(&["(", "foo"]).fails();
+    result.no_stdout();
+    assert_eq!(result.exit_status().code().unwrap(), 2);
+    assert_eq!(
+        result.stderr_str().trim(),
+        "test: missing argument after 'foo'"
+    );
+}
