@@ -1,4 +1,5 @@
 #!/bin/sh
+# spell-checker:ignore uuhelp
 ARG=""
 if test "$1" != "--do-it"; then
     ARG="--dry-run --allow-dirty"
@@ -35,19 +36,20 @@ TOTAL_ORDER=$(echo -e $PARTIAL_ORDER | tsort | tac)
 TOTAL_ORDER=${TOTAL_ORDER#ROOT}
 
 set -e
-for dir in src/uucore_procs/ src/uucore src/uu/stdbuf/src/libstdbuf/; do
+for dir in src/uuhelp_parser/ src/uucore_procs/ src/uucore/ src/uu/stdbuf/src/libstdbuf/; do
     (
         cd "$dir"
         #shellcheck disable=SC2086
-        cargo publish $ARG || true
+#        cargo publish $ARG
     )
+    sleep 2s
 done
 
 for p in $TOTAL_ORDER; do
     (
         cd "src/uu/$p"
         #shellcheck disable=SC2086
-        cargo publish $ARG || true
+        cargo publish $ARG|| true
     )
 done
 
